#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*
 * This program will fill an array with random numbers
 * and then display the contents of the array
 *
 * It then sorts the array by moving the biggest numbers to the
 * bottom of the table until the entire table is sorted.
 *
 */

/* define the size of the table */
#define MAXENT 20

/* some handy functions we will be using */
void display_table(int tbl[], int max); /* display a table */
void sort_biggest(int tbl[], int max); /* sort the table */

main()
{
    int tbl[MAXENT];    /* the table to be filled */
    int i;

    srand(time(NULL));  /* make actual random numbers */

    for (i=0; i < MAXENT; i++)
    { /* fill the table up with random numbers */
        tbl[i] = rand() % 50;  /* with numbers from 0 to 49 */
    }

    /* display the table */
    display_table(tbl, MAXENT);

    /* we now have a random table, sort it */
    printf("Sorting by moving biggest to bottom ...\n");
    sort_biggest(tbl, MAXENT); /* sort moving biggest to bottom */
    display_table(tbl, MAXENT); /* display sorted table */

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
}

/*
 * This program displays the contents of the given table
 */
void display_table(int tbl[], int maxent)
{
    int i;
    for (i=0; i < maxent; i++)
    { /* loop through the array */
        printf("Entry %2d is %2d\n", i, tbl[i]);
        if (i%20 == 19)
        { /* screen full - pause for user */
            printf("Press ENTER to continue: ");
            getchar(); /* wait for user to press ENTER */
        }
    }
}

/*
 * This function sorts a table by moving the biggest numbers to the bottom
 */
void sort_biggest(int tbl[], int maxent)
{
    int bott, row;   /* top of table and current row */
    int tmp;      	/* temporary storage space used for swapping */

    /*
     * Each time we loop through the array, we move the "bottom" of the
     * array up one slot.
     *
     * The first time, we put the biggest number in tbl[maxent-1]
     * The second time, we put the biggest number in tbl[maxent-2], etc.
     * The inner loop has to start with the cell *before* the new 'top'
     * otherwise, you would keep finding the previous biggest ones
     */
    for (bott=maxent-1; bott > 0; bott--)
    { /* the bottom location of the table for this pass thru the array */
        for (row=0; row < bott; row++)
        { /* the cell[s] we are looking at looking for the biggest */
            if (tbl[row] > tbl[bott])
            { /* a new biggest number found - swap it */
                tmp = tbl[bott];         /* save the old biggest */
                tbl[bott] = tbl[row];    /* move new biggest to bottom */
                tbl[row] = tmp;         /* move the old biggest to row */
            }
        } /* end inner loop */
    } /* end outer loop */
}
