#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*
 * This program will fill an array with random numbers
 * and then display the contents of the array
 *
 * It then sorts the array by using the bubble sort
 */

/* define the size of the table */
#define MAXENT 20

/* some handy functions we will be using */
void display_table(int tbl[], int max); /* display a table */
void bubble_sort(int tbl[], int max); /* sort the table */

main()
{
    int tbl[MAXENT];    /* the table to be filled */
    int i;

    srand(time(NULL));  /* make actual random numbers */

    for (i=0; i < MAXENT; i++)
    { /* fill the table up with random numbers */
        tbl[i] = rand() % 50;  /* with numbers from 0 to 49 */
    }

    /* display the table */
    display_table(tbl, MAXENT);

    /* we now have a random table, sort it */
    printf("Sorting using bubble sort ...\n");
    bubble_sort(tbl, MAXENT); /* sort using the bubble sort */
    display_table(tbl, MAXENT); /* display sorted table */

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
}

/*
 * This program displays the contents of the given table
 */
void display_table(int tbl[], int maxent)
{
    int i;
    for (i=0; i < maxent; i++)
    { /* loop through the array */
        printf("Entry %2d is %2d\n", i, tbl[i]);
        if (i%20 == 19)
        { /* screen full - pause for user */
            printf("Press ENTER to continue: ");
            getchar(); /* wait for user to press ENTER */
        }
    }
}

/*
 * This function sorts a table by moving the smallest numbers to the top
 */
void bubble_sort(int tbl[], int maxent)
{
    int bott, row;   /* top of table and current row */
    int tmp;      	/* temporary storage space used for swapping */
    int didswap;    /* flag to indicate we did a swap this pass */

    /*
     * Use the bubble sort to sort the table
     *
     * We go through the table, swapping adjacent cells if needed
     *
     * When we get done with the first pass, the largest number
     * in the table will be at the bottom of the table.
     *
     * For the next pass, we go down to the next to last entry.
     * This continues until either we have sorted the entire table
     * or we don't do a swap during a given pass
     *
     * If we have a pass without swapping, then the array is sorted
     */
    for (bott=maxent-1; bott > 0; bott--)
    { /* the bottom location of the table for this pass thru the array */
        /* now we go down through the table, swapping entries as needed */
        didswap = 0;    /* no swapping yet */
        for (row=0; row < bott; row++)
        { /* the cell[s] we are looking at looking for the smallest */
            if (tbl[row] > tbl[row+1])
            { /* cells out of order - swap them */
                tmp = tbl[row];         /* save bigger cell */
                tbl[row] = tbl[row+1];	/* move the smaller cell up one */
                tbl[row+1] = tmp;       /* move the larger cell down one */
                didswap++;              /* signal that we did a swap */
            }
        } /* end inner loop */
        /* if we didn't do a swap, we can quit since the table is sorted */
        if (didswap == 0) return;
    } /* end outer loop */
}
