#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/*
 * This program will fill an array with random numbers
 * and then display the contents of the array
 *
 * It then sorts the array by moving the smallest numbers to the
 * top of the table until the entire table is sorted.
 */

/* define the size of the table */
#define MAXENT 20

/* some handy functions we will be using */
void display_table(int tbl[], int max); /* display a table */
void sort_smallest(int tbl[], int max); /* sort the table */

main()
{
    int tbl[MAXENT];    /* the table to be filled */
    int i;

    srand(time(NULL));  /* make actual random numbers */

    for (i=0; i < MAXENT; i++)
    { /* fill the table up with random numbers */
        tbl[i] = rand() % 50;  /* with numbers from 0 to 49 */
    }

    /* display the table */
    display_table(tbl, MAXENT);

    /* we now have a random table, sort it */
    printf("Sorting by moving smallest to top ...\n");
    sort_smallest(tbl, MAXENT); /* sort moving smallest to top */
    display_table(tbl, MAXENT); /* display sorted table */

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
}

/*
 * This program displays the contents of the given table
 */
void display_table(int tbl[], int maxent)
{
    int i;
    for (i=0; i < maxent; i++)
    { /* loop through the array */
        printf("Entry %2d is %2d\n", i, tbl[i]);
        if (i%20 == 19)
        { /* screen full - pause for user */
            printf("Press ENTER to continue: ");
            getchar(); /* wait for user to press ENTER */
        }
    }
}

/*
 * This function sorts a table by moving the smallest numbers to the top
 */
void sort_smallest(int tbl[], int maxent)
{
    int top, row;   /* top of table and current row */
    int tmp;      	/* temporary storage space used for swapping */

    /*
     * Each time we loop through the array, we move the "top" of the
     * array down one slot.
     * The first time, we put the smallest number in tbl[0]
     * The second time, we put the smallest number in tbl[1], etc.
     * The inner loop has to start with the cell *after* the new 'top'
     * otherwise, you would keep finding the previous smaller ones
     */
    for (top=0; top < maxent; top++)
    { /* the top location of the table for this pass thru the array */
        for (row=top+1; row < maxent; row++)
        { /* the cell[s] we are looking at looking for the smallest */
            if (tbl[row] < tbl[top])
            { /* a new smallest number found - swap it */
                tmp = tbl[top];         /* save the old smallest */
                tbl[top] = tbl[row];    /* move new smallest to top */
                tbl[row] = tmp;         /* move the old smallest to row */
            }
        } /* end inner loop */
    } /* end outer loop */
}
