#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * This program illustrates how arguments are passed by name and value
 *
 * Both of the functions (foo and bar) return the sum of the squares
 * of the two arguments.  However, the second function (bar) is passed
 * the address of the arguments, which means that bar can (and does)
 * modify their values
 */

int foo(int x1, int x2);    /* args passed by value (normal method) */
int bar(int *x1, int *x2);  /* args passed by addr (values can change) */

main()
{
	int num1 = 1, num2 = 2, x, y;

    printf("num1=%d, num2=%d\n", num1, num2);
   /*
    * First, we will call a routine passing arguments the normal method
    * a copy of the variable is made and passed to the routine
    * This means that the function foo can't modify the actual values
    * of num1 and num2
    */
   printf("Passing arguments by value ...\n");
   x = foo(num1, num2);
   /* show num1 & 2 unchanged */
   printf("num1=%d, num2=%d, x=%d\n", num1, num2, x);
   /*
    * The second function is passed the addresses of num1 and num2
    * The bar routine will modify the values of num1 and num2
    */
   printf("Passing arguments by address (or name) ...\n");
   y = bar(&num1, &num2);
   /* num1 & num2 are changed */
   printf("num1=%d, num2=%d, y=%d\n", num1, num2, y);

    printf("Press ENTER to terminate program: "); /* display another message */
	getchar(); 						/* wait for the user to press ENTER */
}

/*
 * Return the sum of the square of the two arguments
 *
 * This function is passed the values of the arguments
 * so it cannot change the values back in the main routine
 */
int foo(int n1, int n2)
{
   n1 = n1 * n1;    /* square the first argument */
   n2 = n2 * n2;    /* square the second argument */
   return n1+n2;    /* return the sum of the squares of the two arguments */
}

/*
 * Return the sum of the squares of the two arguments
 *
 * This function is passed the address of the arguments
 * so it will change the values back in the main routine
 */
int bar(int *n1, int *n2)
{
   *n1 = *n1 * *n1;     /* square the first argument */
   *n2 = *n2 * *n2;     /* square the second argument */
   return *n1+*n2;      /* return sum of squares of both args */
}

