/*
 *				**** BLACKJACK ****
 *
 * This program plays blackjack
 * Unlike the example in the text book, this one works
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/*
 * The Hand structure represents a person's hand
 * The hand consists of the total points of the hand,
 * the number of cards in the hand, and the cards themselves
 * You can't have more than 10 cards without going over 21
 */
struct Hand
{
	int points;		/* total points in hand */
   int ncards;		/* how many cards */
   int cards[10];	/* actual cards */
};

/*
 * some useful tables for display
 *
 * The Suit array contains representation of the suits
 * The Face array contains representation of the ranks
 */
char Suit[] = "HCDS";
char Face[] = "A23456789TJQK";

/* function prototypes */
void init_deck(int deck[]);
void shuffle(int deck[]);
int play(int Deck[], int topcard);
void display_deck(int deck[]);
void display_card(int card);
void display_hand(struct Hand *hand);
void total(struct Hand *hand);

main()
{
	char buf[BUFSIZ];		/* input buffer for user's responses */
	int Deck[52];			/* the deck of cards */
	int TopCard=52;		/* the top card on the deck */

   /*
    * The deck is represented by an array of 52 integers
    * Each entry represents one card
    *
    * Cards are encoded as follows:
    *		The cards are in order and grouped by suits:
    *			Hearts, Clubs, Diamonds, and Spades
    *		The suits are arranged by rank: Ace, 2, thru 10, J, Q, and K
    *		Aces can be high (11) or low (1)
    *		All face cards (JQK) count as 10
    *
    * This means that the first 13 cards are A thru K of Hearts
    * The next 13 are Clubs, then Diamonds, and finally Spades
    *
    * The variable TopCard indicates which card is on the top of
    * the deck.  After the deck is shuffled, TopCard is set to zero.
    * Each time a card is dealt, the card that TopCard points to is used,
    * then TopCard is incremented.
    *
    * We keep taking cards off the deck until we have dealt more than
    * 30 cards.  Then, before we start the next round, we shuffle the deck.
    * We do this so we don't have to worry about running out of cards,
    * and then having to shuffle the remaining part of the deck, while
    * some cards have already been dealt.
    *
    * The first thing we have to do is to initialize the deck
    * The cards will be in order at the start
    *
    * The topcard is set so that the program will shuffle the
    * cards before staring to play
    */
   printf("Initializing the deck ...\n");
   init_deck(Deck);	/* initialize the deck */
   TopCard = 52;		/* make sure we shuffle the deck */

   /*
    * This is the main loop of the program
    *
    * If we have dealt more than 30 cards, we shuffle the deck
    * and start over.
    *
    * We play one round, and then ask the player if they want to continue
    */
	do
   {
   	/*
       * If we have dealt more than 30 cards with this deck,
       * we will shuffle the entire deck and start again
       */
   	if (TopCard > 30)
      { /* time to shuffle the deck */
	   	printf("Shuffling the deck ...\n");
	   	shuffle(Deck);	/* shuffle the cards */
         TopCard = 0;
      }

      /*
       * Play one round of blackjack
       * This routine will return the new value of TopCard
       */
      TopCard = play(Deck, TopCard);

      /*
       * Now see if the user wants to play another hand
       */
      printf("Play again? (y or n): ");
      gets(buf);
   } while (buf[0] != 'n' && buf[0] != 'N');

   /*
    * All done
    */
   return 0;
}

/*
 *				**** INIT DECK ****
 *
 * This routine will initialize the deck
 *
 * This is done by storing 0-51 in the Deck array
 * This will put all the cards in order, just as if it were brand new
 */
void init_deck(int deck[])
{ /* initialize the deck of cards */
	int i;

   /* initialize the random number generator */
   srand(time(NULL));

   /* store 0-51 in the array */
	for (i=0; i < 52; i++) deck[i] = i;	/* cards are in order */
}

/*
 *				**** SHUFFLE ****
 *
 * This routine will shuffle the deck
 * We shuffle 100 times
 * For each shuffle, we pick two cards at random and switch them
 */
void shuffle(int deck[])
{ /* shuffle the cards */
	int i, a, b, t;

   for (i=0; i < 100; i++)
   { /* shuffle 100 times */
   	a = rand() % 52;	/* first card */
      b = rand() % 52;	/* second card */
      t = deck[a];		/* save first card */
      deck[a] = deck[b];	/* move second to where first was */
      deck[b] = t;			/* move first where second was */
   }
}

/*
 *				**** DISPLAY DECK ****
 *
 * This routine displays the deck
 * It was used while debugging the program
 * You can also use it to cheat. :^)
 *
 * This routine displays 4 lines of 13 cards
 */
void display_deck(int deck[])
{
	int i;

   for (i=0; i < 52; i++)
   {
   	display_card(deck[i]);	/* display this card */
      if ((i%13)==12) printf("\n");	/* display 13 cards per line */
   }
   printf("\n");
}

/*
 *				**** DISPLAY CARD ****
 *
 * This routine will display a given card
 * The card number is passed as an argument
 * By using modulo 13, we can determine the rank of the card
 * By dividing by 13, we can determine the suit
 */
void display_card(int card)
{
	printf(" %c%c", Face[card%13], Suit[card/13]);
}

/*
 *				**** PLAY ****
 *
 * This routine will play a hand of blackjack
 * The dealer and player are each dealt two cards.
 *
 * The player is shown the dealer's card which is face up
 *
 * The player is then shown what they are holding
 * The player is then asked if they want hit (another card)
 * or stand (quit).  This continues until the player stands
 *
 * The dealer then decides how many cards he/she wants
 * The rules the dealer follows are:
 * 	If the player has gone over 21, the dealer stands
 * 	If the dealer has less than 17, the dealer gets hit
 *
 */
int play(int deck[], int topcard)
{
	struct Hand Dealer, Player;	/* dealer & players hands */
   char buf[BUFSIZ];

   /* deal initial hand */
   Player.cards[0] = deck[topcard++];
   Dealer.cards[0] = deck[topcard++];
   Player.cards[1] = deck[topcard++];
   Dealer.cards[1] = deck[topcard++];

   /* initialize the two hands */
   Dealer.ncards = 2;
   total(&Dealer);	/* compute total points for dealer */
   Player.ncards = 2;
   total(&Player);	/* compute total points for player */

   printf("Dealer shows: ");
   display_card(Dealer.cards[1]);	/* display what the dealer has showing */
   printf("\n");

   /*
    * Loop asking the user if they want another card
    * Continue until they are done
    */
   do
   {
   	/* show the player their hand */
	   printf("Player has: ");
	   display_hand(&Player);
      if (Player.points == 21)
      {
      	if (Player.ncards == 2) printf("Blackjack!");
         else printf("You got 21!");
         printf("\tYou win!\n");
         break;
      }
      else if (Player.points > 21)
      { /* player went over ... too bad */
      	printf("You are over!  You lose!\n");
         break;
      }
      /* ask user if they want another card */
      printf("Hit or Stand (H or S)? ");
      gets(buf);
      if (buf[0] == 'h' || buf[0] == 'H')
      { /* player wants hit - deal next card */
      	Player.cards[Player.ncards++] = deck[topcard++];
         total(&Player);	/* calculate total points */
      }
   } while (buf[0] != 's' && buf[0] != 'S');

   /*
    * The dealer decides how many cards he/she wants
    * Keep asking for more cards until they go over 17
    */
   while (1)
   { /* dealer always stops at 17 */
	   printf("Dealer has: ");
	   display_hand(&Dealer);
      if (Dealer.points < 17 && Player.points <= 21)
      { /* Player still in, and Dealer under 17 - take another card */
      	Dealer.cards[Dealer.ncards++] = deck[topcard++];
         total(&Dealer);
      }
      else if (Dealer.points > 21)
      { /* oops! too much */
      	printf("Dealer goes over!\n");
      	break;
      }
      else
      { /* close enough */
      	printf("Dealer stands\n");
         break;
      }
   }

   /* decide who won */
   if (Player.points > 21)
   { /* if the player went over, they lose */
   	printf("You went over.\tYou lose!\n");
   } else if (Dealer.points > 21)
   { /* if the dealer went over, player wins */
   	printf("Dealer went over.\tYou win!\n");
   } else if (Player.points > Dealer.points)
   { /* if player has more points, they win */
   	printf("You beat the dealer.\tYou win!\n");
   }
   else
   { /* anything else, the player lost */
   	printf("Dealer beat you.\tYou lose!\n");
   }

   /* hand over - return new topcard */
   return topcard;
}

/*
 *				**** TOTAL ****
 *
 * This routine computes the total number of points in a hand
 *
 * We first count aces as high, but if that puts us over,
 * then we count them as one instead
 */
void total(struct Hand *hand)
{
	int aces=0;	/* how many high aces in hand */
   int i, val;

   hand->points = 0;	/* reset total points */
   for (i=0; i < hand->ncards; i++)
   { /* go through each card in the hand */
   	/* start with rank (1-13), then convert to 1-10 */
   	val = hand->cards[i] % 13 + 1;	/* rank (1 to 13) */
   	if (val == 1)
      { /* an ace - count as high for now */
      	val = 11;
         aces++;	/* keep track of how many aces we have */
      } else if (val > 10) val = 10;	/* face cards are 10 */
		hand->points += val;	/* add points to hand */
   }

   /*
    * If we went over 21, but we counted an ace as high
    * we keep changing it from ace high to ace low
    */
   while (hand->points > 21 && aces > 0)
   { /* ace high puts us over - count as 1 instead */
   	hand->points -= 10;	/* change from 11 to 1 */
      aces--;	/* one less ace high */
   }

}

/*
 *				**** DISPLAY HAND ****
 *
 * This routine will display a given hand
 * It loops through each card in the hand, and displays it
 *
 */
void display_hand(struct Hand *hand)
{
	int i;

   printf("%2d points: ", hand->points);
   for (i=0; i < hand->ncards; i++) display_card(hand->cards[i]);
   printf("\n");
}

